# Test program for Warren's microcoded CPU
# (c) 2015 Warren Toomey, GPL3 licence
#
main:	li	r7, 0x7FFF	# Initialise the stack pointer
	li	r2, message	# Print out the message
	jsr	puts
	li	r2, 0x0C46	# Store 3142 into r2
	jsr	printn		# Print r2 out in decimal
	li	r2, newline	# Print out a newline
	jsr	puts
	li	r2, 0x0017	# Store 23 into r2
	jsr	printn		# Print r2 out in decimal
	li	r2, newline	# Print out a newline
	jsr	puts
	li	r2, 0x0017	# Store 23 into r2
	addi	r2, r2, 0x0C46	# Add r2 and 3142
	jsr	printn		# Print r2 out in decimal
	li	r2, newline	# Print out a newline
	jsr	puts
end:	jmp	end		# Infinite loop

# Print out a null-terminated string pointed to by r2
#
puts:	lwr	r1, r2		# Load the word that r2 points at
	beqz	r1, putsend	# It's a zero, so stop now
	jsr	putchar		# Print that character out
	inc	r2		# Move up to next character
	br	puts		# and loop back
putsend: rts
	
# Print out the character in r1
#
putchar: lw	r0, 0xC002	# Get the UART status value
	andi	r0, r0, 0x0002	# Keep only the ready bit
	beqz	r0, putchar	# Loop back if not ready
	sw	r1, 0xC001	# Send the character to the UART
	rts			# and return

# Iteratively print r2 out in decimal
#
printn: li	r3, 0x0000	# Push 0x0000 as an end of string indicator
	push	r3
	bgez	r2, ploop1	# Branch if r2 >= 0
	li	r1, 0x002D	# Print out a '-' (0x2D)
	jsr	putchar
	muli	r2, r2, 0xFFFF	# r2 = r2 * -1, i.e. negate the number

# Store the digits of the number one at a time on the stack
ploop1: remi	r1, r2, 0x000A	# r1 = r2 % 10
	addi	r1, r1, 0x0030	# Add ASCII '0' to the digit
	push	r1		# Save this digit on the stack
	divi	r2, r2, 0x000A	# r2 = r2 / 10
	bgtz	r2, ploop1	# Loop back if there is still something in r2

# Now print the digits out in reverse order
ploop2:	pop	r1		# Pull a digit from the stack
	beqz	r1, pnend	# Stop when we reach the 0x0000 marker
	jsr	putchar		# And print the ASCII digit
	br	ploop2		# Branch back to get the next digit
pnend:	rts			# All done

# Data
#
message: ascii	Here are two numbers followed by their sum:
newline: word 0xd
	word 0xa
	word 0x0
