-------------------------------------------------------
-- Design Name : rom
-- File Name   : rom.vhd
-- Function    : ROM Using readmemb
-- Coder       : Deepak Kumar Tala (Verilog)
-- Translator  : Alexander H Pham (VHDL)
-------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;
use std.textio.all;

entity rom is
    generic (filename : string :="generic.list";
             ADDR_WIDTH :    integer := 8;
             DATA_WIDTH :    integer := 8
    );
    port (ce    :in  std_logic;                   	       -- Chip enable
        read_en :in  std_logic;                     	       -- Read enable
        address :in  std_logic_vector (ADDR_WIDTH-1 downto 0); -- Address input
        data    :out std_logic_vector (DATA_WIDTH-1 downto 0)  -- Data output
      );
end entity;

architecture behavior of rom is
    -- ROM block
    type ROM is array (integer range <>) of
				std_logic_vector (DATA_WIDTH-1 downto 0);
    signal mem : ROM (0 to 2**ADDR_WIDTH-1);
    
    -- Instructions to read a text file into ROM --
    procedure Load_ROM (signal data_word :inout ROM) is
        -- Open File in Read Mode
        file romfile   :text open read_mode is filename;
        variable lbuf  :line;
        variable i     :integer := 0;
        variable fdata :std_logic_vector (DATA_WIDTH-1 downto 0);
    begin
        while not endfile(romfile) loop
            -- read data from input file
            readline(romfile, lbuf);
            read(lbuf, fdata);
            data_word(i) <= fdata;
            i := i+1;
        end loop;
    end procedure;
    
begin
    -- Procedural Call --
    Load_ROM(mem);
    data <= mem(to_integer(unsigned(address)))
      when (read_en = '1' and ce = '1') else (others=>'0');
end architecture;

-------------------------------------------------------
-- Test bench for ROM
-------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity rom_test is
    generic (DATA_WIDTH :integer := 8;
             ADDR_WIDTH :integer := 9
    );
end entity;

architecture test of rom_test is
    signal address :std_logic_vector (ADDR_WIDTH-1 downto 0) := (others=>'0');
    signal read_en :std_logic := '0';
    signal ce      :std_logic := '0';
    signal data    :std_logic_vector (DATA_WIDTH-1 downto 0) := (others=>'0');
    signal clk     :std_logic := '0';
    
    component rom is
        generic (filename : string :="memory.list";
            	 ADDR_WIDTH :integer := ADDR_WIDTH;
            	 DATA_WIDTH :integer := DATA_WIDTH
        );
        port (ce    :in  std_logic;                     -- Chip enable
            read_en :in  std_logic;                     -- Read enable
            address :in  std_logic_vector (ADDR_WIDTH-1 downto 0); -- Address input
            data    :out std_logic_vector (DATA_WIDTH-1 downto 0)  -- Data output
          );
    end component;
begin
    clk <= not clk after 10 ns;
    process begin
        for i in integer range 0 to (2** ADDR_WIDTH -1) loop
            address <= conv_std_logic_vector(i, ADDR_WIDTH);
            read_en <= '1';
            ce      <= '1';
            -- Old code used to toggle these lines
            -- read_en <= '1' after 5 ns, '0' after 15 ns;
            -- ce      <= '1' after 5 ns, '0' after 15 ns;
            wait for 20 ns;
        end loop;
        report "rom test finished" severity failure;
    end process;
    
    Inst_Rom : rom port map (
        address => address, -- Address input
        data    => data,    -- Data output
        read_en => read_en, -- Read enable
        ce      => ce       -- Chip enable
      );
end architecture;
